from os.path import exists

import gnupg
from flask import request, Blueprint, Response, redirect
from htmlpy import *

from config import CONFIG, colors
from html_fn import buildSite
from msg import encrypt, save_message
from notification import notify
from fn import is_browser

main_pages = Blueprint("main", __name__)

# Color Scheme
@main_pages.route("/color")
def color():
    if colors() is None:
        return ""
    colors_p = [Heading(1, "Color 0")]
    for i in range(0, 16):
        c = colors()["colors"][f"color{i}"]
        colors_p.append(
            Heading(
                1, f"Color {i+1}", global_attr=GlobalAttributes(style=f"color: {c}")
            )
        )
    site = buildSite([], "Colors")

    bg = colors()["special"]["background"]
    fg = colors()["special"]["foreground"]
    cur = colors()["special"]["cursor"]
    site.body = Body(
        [Div([colors_p], global_attr=GlobalAttributes(css_class="container"))],
        global_attr=GlobalAttributes(
            style=f"background: {bg}; color: {fg}",
            css_class="justify-content-center text-center",
        ),
    )

    return site.to_code()


# Mirrors
@main_pages.route("/mirrors.txt", methods=["GET"])
def mirrors():
    # TODO : Autogenerate GPG Sign Message
    if exists("/config/mirrors.txt"):
        txt = open("/config/mirrors.txt").read()
        if is_browser(request):
            site = buildSite(
                Div(
                    f"<pre>{txt}</pre>",
                    global_attr=GlobalAttributes(style="margin: 25px;"),
                ),
                "Mirrors",
            )

            return site.to_code()

        return txt
    else:
        return Response(response="", status=404)


# Message Sending Page
@main_pages.route("/message", methods=["GET", "POST"])
def send_message():
    if request.method == "POST":
        # If POST handle message
        msg = request.form["message"]
        name = request.form["msg_name"]
        if msg is not None:
            cipher = encrypt(msg)
            save_message(cipher, name)
            notify(f"New Message from {name}")
            return redirect(request.url_root)
    else:
        # Return Message Form
        return buildSite(
            [
                Div(
                    [
                        Heading(1, "Message"),
                        LineBreak(),
                        Form(
                            destination=request.url,
                            inner=[
                                Input(
                                    "",
                                    placeholder="Name",
                                    name="msg_name",
                                    global_attr=GlobalAttributes(
                                        css_class="form-control bg-dark text-white",
                                        style="margin-bottom: 15px",
                                    ),
                                ),
                                TextArea(
                                    inner="",
                                    placeholder="Message",
                                    name="message",
                                    global_attr=GlobalAttributes(
                                        css_class="form-control bg-dark text-white",
                                        style="margin-bottom: 15px;",
                                    ),
                                ),
                                Input(
                                    type=InputType.Submit,
                                    value="Send Message",
                                    name="submit",
                                    global_attr=GlobalAttributes(
                                        css_class="btn btn-danger text-white text-decoration-none"
                                    ),
                                ),
                            ],
                        ),
                    ],
                    global_attr=GlobalAttributes(
                        css_class="container", style="margin-top: 25px"
                    ),
                )
            ],
            "Send a message",
        ).to_code()


# Public Key Page
@main_pages.route("/public_key", methods=["GET"])
def public_key():
    try:
        ret = open("/config/pub.key").read()

        pgp = gnupg.GPG()
        pgp.import_keys(open("/config/pub.key").read())
        key_id = (
            str(pgp.list_keys()[0]["uids"][0]).replace("<", "[ ").replace(">", " ]")
        )

        if is_browser(request):
            # If request is from common browser return stylized html
            return buildSite(
                [
                    Div(
                        Div(
                            [
                                Bold("To Import: "),
                                Span(
                                    f'curl -sL "{request.base_url}"|gpg --import',
                                    global_attr=GlobalAttributes(
                                        style="display: block;font-family: monospace,monospace;margin-top: 10px; font-size: 20px;overflow-wrap: break-word;"
                                    ),
                                ),
                            ],
                            global_attr=GlobalAttributes(css_class="alert alert-info"),
                        ),
                        global_attr=GlobalAttributes(
                            css_class="container", style="margin-top: 25px"
                        ),
                    ),
                    Heading(
                        4,
                        key_id,
                        global_attr=GlobalAttributes(
                            css_class="container card",
                            style="padding-top: 10px; padding-bottom: 10px; background: black; margin-bottom: 15px;",
                        ),
                    ),
                    Div(
                        Paragraph(ret.replace("\n", "<br>")),
                        global_attr=GlobalAttributes(
                            css_class="container card bg-primary"
                        ),
                    ),
                ],
                "Public Key",
                disable_color=True,
                shadow=False,
            ).to_code()

        # Return raw key
        resp = Response(response=ret, status=200, mimetype="application/pgp-keys")
        return resp
    except:
        # If key does not exist return error
        return Response(response="", status=502)


# Contact
def build_contact_block():
    return Div(
        [
            Heading(1, "Contact"),
            ThematicBreak(),
            [
                Link("/public_key", "My PGP Key"),
                LineBreak(),
                Link("/message", "Write a message"),
                LineBreak(),
                LineBreak(),
            ]
            if exists("/config/pub.key")
            else None,
            Link(f"mailto:{CONFIG['email']}", CONFIG["email"]),
            LineBreak(),
        ],
        global_attr=GlobalAttributes(css_class="container border-dark"),
    )


# Donations
def build_donation_block():
    return Div(
        [
            Heading(1, "Donation"),
            ThematicBreak(),
            Paragraph(
                [
                    Bold("Monero: "),
                    Span(
                        CONFIG["xmr_address"],
                        global_attr=GlobalAttributes(
                            style="color: orange;overflow-wrap: break-word;"
                        ),
                    ),
                ]
            )
            if "xmr_address" in CONFIG
            else None,
        ],
        global_attr=GlobalAttributes(css_class="container", style="margin-top: 20px"),
    )


# Basic Information
def build_information_block():
    return Div(
        [
            Image(
                "/assets/me",
                200,
                200,
                "Me",
                global_attr=GlobalAttributes(css_class="rounded"),
            ),
            LineBreak(),
            LineBreak(),
            Heading(1, CONFIG["name"]),
            ThematicBreak(),
        ],
        global_attr=GlobalAttributes(
            css_class="container border-dark", style="margin-top: 20px"
        ),
    )


# Main
@main_pages.route("/", methods=["GET"])
def index():
    return buildSite(
        [build_information_block(), build_contact_block(), build_donation_block()],
        "About Me",
    ).to_code()
